//javascriptねこらいぶらりーver.1.0
//クロスブラウザで動くjavascriptクラス群を提供します。
//モジュールを必要に応じてインポートすることにより使用します。
//モジュールは、NL_??????(モジュール名).js というファイルに定義されていて、
//その実体は NL_?????? という名前のクラスです。
//import(className)メソッドにより、そのモジュールを読み込みクラスを使用可能にできます。
//この際同時に、このクラスの動作に必要な他のクラスも自動で読み込まれます。

/*[使用法]
	ライブラリーファイルがあるフォルダ(NekoLibrary)のhtmlからのパスが
	"include/NekoLibrary"であると仮定します。（適宜置き換えてください）

	※(注意)3.と4.の処理は同じ<script>タグ内に書かないで別の<script>タグ内に書いてください。

	htmlのhead中で、
	
	1.まず本体 NekoLibrary.js を読み込みます
	(例)
		<script src="include/NekoLibrary/NekoLibrary.js"></script>
	2.ライブラリーがあるフォルダを指定して NL_importStart(folder) を呼びインポートの準備をします。
	(例)
		<script type="text/javascript">
			NL_importStart('include/NekoLibrary/lib');	//必ず呼んでください
		</script>
	3.使いたいモジュール NL_?????? を、NL_import(className) を用いて読み込みます。
	(例)
		<script type="text/javascript">
			NL_import('NL_Window');			//インポート
		</script>

	最後にhtmlのbodyの最初で

	4.NL_importFinish() を呼んでインポートがすべて終了したことを通知し、モジュールを構成します。
	(例)
		<script type="text/javascript">
			NL_importFinish();			//必ず呼んでください
		</script>

	(以上)
	※ライブラリーが使用できるのは、NL_importFinish()を呼んだ行より後ろです。

*/

/*[使用例]
	<head>
		<script src="include/NekoLibrary/NekoLibrary.js"></script>
		<script type="text/javascript">
			NL_importStart('include/NekoLibrary/lib');	//必ず呼んでください
			NL_import('NL_Window');			//インポート
		</script>
	</head>
	<body>
		<script type="text/javascript">
			NL_importFinish();			//必ず呼んでください
		</script>
		<!--以後ライブラリが使用できます-->

	</body>
*/

/*[技術情報]
	・ねこらいぶらりーのフレームワーク
		　ねこらーぶらりーは、使いたいクラスモジュールをインポートするだけで
		そのモジュールに必要な他のファイルを自動的に読み込んでいます。
		
		　この処理は、各クラスモジュールが必要な他のモジュールをNL_import()ですべて呼び出すことにより行われています。
		この時同じファイルを何度も読み込まないように管理されています。これは、NL_importClass[]という連想配列に既に読み込んだ
		モジュールを記録することにより実現されています。

		　NL_importは、<script>タグをdocument.writeすることにより動的にscriptファイルを読み込んでいます。
		ただし、調べてみたところ実際にこれらのファイルが読み込まれるのは<script>タグをdocument.writeしているスクリプトが終了（EOFor</script>）したときです。
		これは、あるライブラリクラス中で他のライブラリクラスを参照できないという問題を発生させます。
		このことが特に問題になるのは、クラスの継承を行うときとプロトタイプ変数を初期化するときです。
		この解決のため、クラスの継承及びプロトタイプ変数の初期化の処理を分離して後で実行する必要がありました。
		その結果、<body>中でNL_importFinish()を呼ぶ必要がある構造になりました。
		<head>の最後でなく、<body>中としたのは、document.bodyが取得できないと困るという問題もあったためです。

	・アクセス制御
		　ねこらーぶらりーで定義されるクラスにおいては、変数に対して命名規則によりアクセスの可否を明示しています
		__で始まる変数名：
			privateです。このクラス内のみからアクセスしてください。
		_で始まる変数名：
			protectedです。このクラスもしくはその派生クラス内のみからアクセスしてください。
		それ以外：
			publicです。どこからでもアクセスしてかまいません。

		　上記のアクセス制御は規則に過ぎずフレームワークとして実際のアクセスを規制しているわけではありません。
*/

////////////////////////////////////////////////////////////////////
/****** global function ******
	void NL_importStart();			//ライブラリを読み込む準備をします。
	void NL_importStart(string folder);	//インポートファイルがあるフォルダを指定し、ライブラリを読み込む準備をします。
	void NL_importFinish();			//ライブラリのインポートがすべて終了したことを通知します。

	void NL_import(string className);	//class名のクラスを読み込みます。
	void NL_import(string className,string subClassName);
						//class名のクラスを読み込み、subClassの基底クラスとして登録します。
	void NL_enrollAlias(string className,string dstName,string srcName);
						//classNameをもつクラスにおいて変数の別名を登録します
	void __NL_setAlias(string className);	//classNameをもつクラスに対しNL_enrollAlias()で登録された別名をすべて実際に作成します。
	void __NL_initAll(int no);		//NL_importFinish()からのみ呼ばれる、継承順序順に初期化を行うための関数です

	void NL_inherit(obj subClass,obj superClass);
						//superクラスで示されるクラスをsubclassで示されるクラスの基底クラスとして登録します

	void NL_error(string title,string msg,int skip);
						//
	string NL_getInfo(object obj,int maxlevel,int level);
						//objのmaxlevel代までの子孫要素のすべてを表示するための文字列を返します。
	string NL_getInfo(object obj,int maxlevel,int level,string[] ignore);
						//objのmaxlevel代までの子孫要素のうちignore以下を除いたすべてを表示するための文字列を返します。
	string NL_htmlFilter(string str);	//\n,\tをhtmlに表示できるよう、スペースと改行に置き換えます。
	string NL_tagEscape(string str);	//htmlの<,>,&,を&lt;,&gt;,&amp;に置き換えます。
	string NL_strRepeat(string str,int cnt);//strをcnt回繰り返した文字列を返します

	void NL_copyUndefProperties(obj src,obj dest);
						//コピー先destで未定義のクラスプロパティにsrcのクラスプロパティを一括コピーします。
	bool NL_isValueOf(object obj,object value);
						//objの連想配列(子要素)の値としてvalueが存在するか判定します。
	object NL_getOffspring(object obj,object item,int cnt);
						//
	string NL_digestOf(object obj);		//objが関数の時、関数の定義文字列をかえします。
	bool NL_isFunction(object obj);		//objが関数かどうか返します。
	bool NL_isObject(obj);			//objがオブジェクトかどうか返します。
	bool NL_isNaN(obj);			//objが数とみなせないかどうか返します。
************************/

////////////////////////////////////////////////////////////////////
//****** グローバル変数 ******
NL_importClass=[];
NL_inheritRelation=[];
NL_importFolder='lib';
NL_importComplete=true;

NL_copyBuffer=[];

NL_browser = (window.opera?'opera':(document.all?'ie':(window.controllers?'nn6':(document.layers?'nn4':navigator.userAgent.toLowerCase().indexOf("safari")?'saf':'unknown'))));
NL_isIE5 = (navigator.appName.indexOf('Microsoft Internet Explorer')>=0 && document.getElementById)?1:0;

//****** グローバル関数 ******
//クラスインポート用

function NL_importStart(folder){
	NL_importComplete=true;
	if(typeof(folder)=='string') NL_importFolder=folder;
}
function NL_import(className,subClassName){
	if(className.substring(0,3)=='NL_') NL_importComplete=false;	

	if(subClassName) NL_inheritRelation[subClassName]=className;

	if(typeof(NL_importClass[className])=='undefined'){
		NL_importClass[className]=null;
		document.write('<script src='+NL_importFolder+'/'+className+'.js></script>');
	}
}
function NL_importFinish(){
	if(!NL_importComplete){
		alert('[NekoLibrary: fatal error]NL_importとNL_importFinishを同じ<script>タグ内で呼ばないでください');//IEではこのメッセージがでない。
	}
	//インポートが成功したか確認
	for(var className in NL_importClass){
		var msg='[NekoLibrary: fatal error]クラス'+className+'は読み込まれませんでした。ファイルが存在するか確認してください。';
		eval("if(typeof("+className+")=='undefined') alert('"+msg+"');");
	}

	__NL_initAll(1);//継承
	__NL_initAll(2);//初期化関数をコール
	__NL_initAll(3);//もう一度継承(初期化関数反映)
}
function __NL_initAll(no){
	var level=1;
	var end=false;
	while(!end){
		end=true;
		for(var className in NL_importClass){
			if(!NL_getOffspring(NL_inheritRelation,className,level))
			{
				if(!NL_getOffspring(NL_inheritRelation,className,level-1)) continue;
				if(!className.substring(0,3)=='NL_') continue;
				var superClassName=NL_inheritRelation[className];
				switch(no){
				case 1:
					if(superClassName){
						eval('if(!'+className+'.__initProperty)'+className+'.__initProperty=function(){};');
						eval('NL_inherit('+className+','+superClassName+');');
					}
					__NL_setAlias(className);
					break;
				case 2:
					eval('if('+className+'.__initProperty)'+className+'.__initProperty();');
					break;
				case 3:
					if(superClassName)
						eval('NL_inherit('+className+','+superClassName+');');
					break;
				}
			}
			else end=false;
		}
		level++;
	}

}
function __NL_setAlias(className){
	//importFinish時に呼ばれ、
	//NL_enrollAliasで登録されていたエイリアスを実際にセットします。
	if(!NL_copyBuffer[className]) return;
	for(var dst in NL_copyBuffer[className]){
		var src=NL_copyBuffer[className][dst];
		var stmt=className+'.prototype.'+dst+'='+className+'.prototype.'+src+';';
		eval(stmt);	
	}
}
function NL_enrollAlias(className,dstName,srcName)
{
	//importファイルの中でのみ使用してください。
	if(typeof(NL_copyBuffer[className])=='undefined')
		NL_copyBuffer[className]=[];
	NL_copyBuffer[className][dstName]=srcName;
}
//****** 継承用
function NL_inherit(subClass, superClass){
	//プロパティとメソッドのコピー
	NL_copyUndefProperties(superClass.prototype , subClass.prototype);
	
	//スタティックなプロパティとメソッドのコピー
	NL_copyUndefProperties(superClass, subClass);
}

//******* デバッグ用
function NL_error(title,msg,skip){
	if(!skip) skip=0;
	var func=NL_error.caller;
	if(!func) return null;
	for(var n=0;n<skip;n++) func=func.caller;
	var str=title+' : '+msg+" in "+NL_digestOf(func);
	while(func.caller){
		str+='\n from '+NL_digestOf(func);
		func=func.caller;
	}
	alert(str);
	return null;
}
function NL_typeError(arg,type,skip){
	if(!skip) skip=0;
	return NL_error('type error','type of argument "'+arg+'" should be '+type+'.',skip+1)
}

//****** 状態表示用
function NL_getInfo(obj,maxlevel,level,ignore){
	var str="";
	if(!obj) return;
	if(!level) level=0;
	if(!maxlevel)maxlevel=0;
	if(level>maxlevel) return "";
	for(var key in obj){
		if(NL_isValueOf(ignore,key)){
			str+=NL_strRepeat("　　　　",level)+key+"    =>    省略\n";
		}
		else if(obj[key]){
			str+=NL_strRepeat("　　　　",level)+key+"    =>    "+NL_digestOf(obj[key])+"\n";
			str+=NL_getInfo(obj[key],maxlevel,level+1,ignore);
		}
		else{
			str+=NL_strRepeat("　　　　",level)+key+"    =>    "+obj[key]+"\n";
		}
	}
	return str;
}

//****** 文字列操作用
function NL_htmlFilter(str){
	str=str.replace(/\t/g,'　　');
	str=str.replace(/\n/g,'<br />');
	return str;
}
function NL_tagEscape(str){
	if(typeof(str)!='string') return null;
	str=str.replace(/&/g,"&amp;");
	str=str.replace(/</g,"&lt;");
	str=str.replace(/>/g,"&gt;");
	return str;
}
function NL_strRepeat(str,cnt){
	var rtn="";
	for(var n=0;n<cnt;n++)
	{
		rtn+=str;
	}
	return rtn;
}
function NL_removeSpecialChar(str){
	str=str.replace(/\n/g,'');
	str=str.replace(/\r/g,'');
	str=str.replace(/\t/g,'');
	return str;
}
//******* 一般関数
function NL_copyProperties(src, dest)
{
	for (var prop in src) {
		dest[prop] = src[prop];
	}
}
function NL_copyUndefProperties(src, dest)
{
	//srcの変数をdestの変数にコピーします。
	//ただしdestですでに同名の変数が定義されている場合はコピーしません。
	for (var prop in src){
		if (typeof(dest[prop]) == "undefined")
			dest[prop] = src[prop];
	}
}
function NL_isValueOf(obj,value){
	for(var key in obj){
		if(obj[key]==value) return true;
	}
	return false;
}
function NL_getOffspring(obj,item,cnt){
	if(cnt==0) return item;
	if(obj[item]) return NL_getOffspring(obj,obj[item],cnt-1)
	return null;
}
function NL_digestOf(obj){
	var val=obj+""; //文字列に変換
	val=val.replace('\n','');
	if(val) var match=val.match(/^function\s?\(.*\)/i);
	if(!match) match=val.match(/^function\s[\w_]+\(.*\)/i);
	if(match)
		return match;
	else	
		return val;
}
//******* 型判別関数
function NL_isFunction(obj){
	var val=obj+""; //文字列に変換
	val=val.replace('\n','');
	if(val) var match=val.match(/^function\s?\(.*\)/i);
	if(!match) match=val.match(/^function\s[\w_]+\(.*\)/i);
	if(match)
		return true;
	else	
		return false;
}
function NL_isObject(obj){
	return (typeof(obj)=='object'&&obj!=null);
}
function NL_isNaN(obj){
	return (isNaN(obj)|| obj==null);
}
