NL_import('NL_DebugLog');
NL_import('NL_Color');
NL_import('NL_RealRect','NL_DelaxRect');
NL_importComplete=true;

/*[使用法]
	NL_DelaxRectクラスはコンストラクタnew NL_DelaxRect()により
	絶対座標に配置される矩形エレメントを生成します。
	エレメントは生成と同時にドキュメントに挿入されます

	NL_DelaxRectのプロパティについて：
	プロパティの値を変更した後、updateメソッドを実行する事で表示を更新します。
*/

//[使用例] example_NL_DelaxRect.htm

//[作者] Mattsun

//[メンバ一覧]
/****** global function ******
	NL_DelaxRect(left,top,width,height);		//
	NL_DelaxRect(left,top,width,height,parent);	//
************************/

/****** property *******
	NL_Color	lineColor;	//
	NL_Color	fillColor;	//
	bool		hasLine;	//
	bool		hasFilling;	//
	int		lineWidth;	//
	string		lineStyle;	//
	bool		shouldApplyStyle	: NL_DelaxRectで定義されたスタイル関連プロパティを有効にする場合true。
									  初期値は直接new NL_DelaxRectでインスタンスが生成された場合はTrue,
									  それ以外の方法で新規生成された場合はfalse,copyToメソッドを使用した場合は
									  コピーもとに準拠、となります。このプロパティは生成後に変更可能です。
************************/

/****** method *******
	void		update()	//位置・サイズ・色およびスタイルのプロパティの値を描画に反映させます(オーバーライド)
	NL_DelaxRect 	copyTo(parent);	//複製を作ります(オーバーライド)

************************/

//[プログラム本体]
NL_DelaxRect.createWithElement = function(HTMLBlockElement){
	var R 		= new NL_DelaxRect(0,0,0,0);
	var Div 	= HTMLBlockElement;
	Div.style.position = "absolute";
	R.remove();
	R.div			= Div;
	R.moveTo(R.getLeft(),R.getTop());
	R.resizeTo(R.getWidth(),R.getHeight());
	R.shouldApplyStyle = false;
	return R;
}
NL_DelaxRect.__initProperty = function(){
	NL_DelaxRect.__count				= 0;
	NL_DelaxRect.prototype.hasLine		= true;
	NL_DelaxRect.prototype.hasFilling	= true;
	NL_DelaxRect.prototype.lineWidth	= 1;
	NL_DelaxRect.prototype.lineStyle	= "solid"
	NL_DelaxRect.prototype.lineColor	= new NL_Color(64,64,64);
	NL_DelaxRect.prototype.fillColor	= new NL_Color(200,200,200);
	NL_DelaxRect.prototype.shouldApplyStyle		= true;
};
//********** constructor **********//
function NL_DelaxRect(left,top,width,height,parent){
	NL_RealRect.apply(this,arguments);
	this.div.id = "NL_DeluxRect_defID_" + NL_DelaxRect.__count;
	NL_DelaxRect.__count++;
}
//****** method *******//
NL_DelaxRect.prototype.update = function(){
	//色およびスタイルの設定を描画に反映させます//
	if(this.shouldApplyStyle){
		NL_DebugLog.add("DeluxRect","update()");
		
		this._setBorder(this.lineStyle,this.lineWidth * this.hasLine,this.lineColor.toString());
		NL_DebugLog.add("DeluxRect","@update : this.hasFilling = " + this.hasFilling + " : " + this.fillColor);
		if(this.hasFilling){
			
			this._setBackColor(this.fillColor.toString());
		}else{
			this._setBackColor();
		}
	}//end if
	
	NL_RealRect.prototype.update.call(this);

};
NL_DelaxRect.prototype.copyTo = function(parent){
	//複製を返します//
	if(!parent) parent = document.body;
	var Elem = new NL_DelaxRect(this.getLeft(),this.getTop(),this.getWidth(),this.getHeight(),parent);
	var Div = Elem.div;
	NL_copyProperties(this,Elem);
	Elem.div = Div;
	Elem.setVisibility(true);
	
	Elem.lineColor	= this.lineColor.copy();
	Elem.fillColor	= this.fillColor.copy();
	NL_DebugLog.add("DeluxRect","@copyFrom :" + this + " , To : " + Elem + " , Is CopyFrom == CopyTo ? : " + (this.div == Elem.div).toString());
	return Elem;
};
NL_DelaxRect.prototype.toString = function(){
	return "[Object DeluxRect : id = " + this.div.id
				+ "\n  left=" + this.getLeft()
				+ "\n  top=" + this.getTop()
				+ "\n  width=" + this.getWidth()
				+ "\n  height=" + this.getHeight()
				+ "\n  hasLine=" + this.hasLine + "(" + this.lineWidth + " " + this.lineStyle + " " + this.lineColor + ")"
				+ "\n  hasFilling=" + this.hasFilling + "(" + this.fillColor + ")"
				+ "\n  parent=" + this.div.parentNode + "]";
}

//******************** /NL_DelaxRect class ********************//

