NL_import('NL_DebugLog');
NL_import('__NL_Dom');
NL_import('NL_Rect','NL_RealRect');
NL_importComplete=true;
//******************** NL_RealRect class ********************
//htmlエレメントとして自由に配置可能であるabsoluteなdivのクロスブラウザなラッパークラスです。
//NL_Rect のメソッドを用いて移動、リサイズ等できますが、
//その変更を表示に反映させるには、update()メソッドを呼ぶ必要があります。
//__real,__visible,zOrder等のプロパティを直接変更したときにもupdate()メソッドを呼ぶ必要があります。
//通常は、setReality()等のメソッドを使用した方が便利です。
//オーバーライドによりgetLeft()等は内部変数を返すのでなく、
//divに設定されている実際の値を返します。

/*[使用法]
	
*/

//[使用例] example_NL_DelaxRect.htm

//[作者] Gen

//[メンバ一覧]
/****** global function ******
	NL_RealRect(HTMLElement elem);	//
	NL_RealRect(HTMLElement elem,HTMLElement parent);
					//
	NL_RealRect(int left,int top,int width,int height);
					//
	NL_RealRect(int left,int top,int width,int height,HTMLElement parent);
					//
************************/
/****** property *******
	HTMLElement	div;		//
	HTMLElement	pDiv;		//

	int		zOrder;		//
	bool		__real;		//
	bool		__visible;	//
************************/

/****** method *******
	int		getLeft();		//四角形の左辺の座標を返します(オーバーライド)
	int		getTop();		//四角形の上辺の座標を返します(オーバーライド)
	int		getWidth();		//四角形の幅を返します(オーバーライド)
	int		getHeight();		//四角形の高さを返します(オーバーライド)

	void		getReality();
	void		setReality(bool real);
						//四角形の存在・非存在を即座に切り替えます
	void		getVisibility();
	void		setVisibility(bool visible);
						//四角形の表示・非表示を即座に切り替えます
	void		setZOrder(int zOrder);	//四角形の重なり順を即座に切り替えます

	void		setClip();		//四角形の表示領域を現在の四角形の大きさにあわせます。
	void		setClip(int top,int right,int bottom,int left);
						//四角形の表示領域を指定します
	void		_setBackColor();	//四角形の背景を透明にします。
	void		_setBackColor(string col);
						//四角形の背景色を指定します。
	void		_setBorder(string lineStyle,int lineWidth,string lineColor);
						//四角形のボーダーを設定します。

	NL_RealRect 	copyTo(parent);		//複製を作ります
	void		remove();		//html要素を消去します。

	void		update();		//内部状態を一括して反映させます。
	void		_move();		//実際に四角形を移動します。
	void		_resize();		//実際に四角形をリサイズします。
************************/

//[プログラム本体]
//****** property *******
NL_RealRect.prototype.zOrder		= 0;
NL_RealRect.prototype.__real		= true;
NL_RealRect.prototype.__visible		= true;

//********** constructor **********
function NL_RealRect(arg1,arg2,arg3,arg4,arg5){
	var parent;
	if(arguments.length>=4)	parent=arg5;
	else			parent=arg2;
	if(!parent || !parent.appendChild) parent=document.body;
	if(NL_browser =='nn4'){
		this.div=parentDiv?(new Layer(width,parentDiv)):(new Layer(width));
		if(height>0) this.div.resizeTo(width,height);
		this.div.moveTo(left,top);
	}
	else{
		this.div = document.createElement("div");
		this.div.style.position = "absolute";
		parent.appendChild(this.div);
	}

	this.div.admin = this;	//動的属性
	this.pDiv=parent;

	NL_Rect.apply(this,arguments);
	this.update();
}
//****** method *******
NL_RealRect.prototype.update = function(){
	//位置・サイズ等のプロパティの値を描画に反映させます//
	this.setReality(this.__real);
	this.setVisibility(this.__visible);
	this.setZOrder(this.zOrder);
	this._move();
	this._resize();
};

//*** 座標関連 ***
NL_RealRect.prototype.getLeft = function(){
	return __NL_Dom.getLeft(this.div);
};

NL_RealRect.prototype.getTop = function(){
	return __NL_Dom.getTop(this.div);
};
NL_RealRect.prototype.getWidth = function(){
	return __NL_Dom.getWidth(this.div);
};
NL_RealRect.prototype.getHeight = function(){
	return __NL_Dom.getHeight(this.div);
};

//*** 移動リサイズ関連 ***
//リサイズを実際に反映
NL_RealRect.prototype._resize = function(){
	//警告：まだクロスブラウザになっていません。
	//ブラウザによってボーダーの扱いに差があります。
	//IE		:width,heightはボーダーを含みます。（意図する動作）
	//opera8,ff	:width,heightはボーダーを含みません。サイズはwidth,heightより大きくなります（意図しない動作）
	//example_NL_Rect.htmを用いて確認してください
	if(NL_browser =='nn6'){
		this.div.style.width =this._width +'px';
		this.div.style.height=this._height+'px';
	}
	else if(NL_browser =='nn4'){
		this.div.resizeTo(this._width,this._height);
	}
	else if(NL_browser =='ie'||NL_browser =='opera'){
		this.div.style.pixelWidth  = this._width;
		this.div.style.pixelHeight = this._height;
	}
	this.setClip();
};
//移動を実際に反映
NL_RealRect.prototype._move = function(){
	if(NL_browser =='nn6'||NL_browser =='opera'){
		this.div.style.left=this._left+'px';
		this.div.style.top =this._top +'px';
		return;
	}
	if(NL_browser =='nn4'){ this.div.moveTo(this._left,this._top); return; }
	if(NL_browser =='ie'){
		this.div.style.pixelLeft=this._left;
		this.div.style.pixelTop =this._top;
		return;
	}
};
//*** 表示関連 ***

//有効
NL_RealRect.prototype.getReality = function(){
	return this.__real;
};
NL_RealRect.prototype.setReality = function(real){
	this.__real=real;
	if(real) this.div.style.display="block";
	else this.div.style.display="none";
};
//可視
NL_RealRect.prototype.setVisibility = function(visible){
	this.__visible=visible;
	if(visible) this.div.style.visibility="visible";
	else this.div.style.visibility="hidden";
};
//z-order変更
NL_RealRect.prototype.setZOrder = function(zOrder){
	this.zOrder=zOrder;
	if(NL_browser =='nn6'||NL_browser =='ie'||NL_browser =='opera')
		this.div.style.zIndex=zOrder;
	else if(NL_browser =='nn4')
		this.div.zIndex=zOrder;
};

//クリッピング
NL_RealRect.prototype.setClip = function(top,right,bottom,left){
	if(NL_isNaN(top)) top=0;
	if(NL_isNaN(right)) right=this.getWidth();
	if(NL_isNaN(bottom)) bottom=this.getHeight();
	if(NL_isNaN(left)) left=0;

	if(NL_browser =='nn6'||NL_browser =='ie'||NL_browser =='opera'){
		this.div.style.clip='rect('+top+'px '+right+'px '+bottom+'px '+left+'px)';
		return;
	}
	if(NL_browser =='nn4'){
		this.div.clip.top   =top;
		this.div.clip.right=right;
		this.div.clip.bottom=bottom;
		this.div.clip.left =left;
		return;
	}
};
//ボーダー
NL_RealRect.prototype._setBorder = function(lineStyle,lineWidth,lineColor){
	//指定しない項目は変更しない
	if(lineStyle)this.div.style.borderStyle = lineStyle;
	if(!NL_isNaN(lineWidth))this.div.style.borderWidth = lineWidth + "px";
	if(lineColor) this.div.style.borderColor = lineColor;
};
//背景色変更
NL_RealRect.prototype._setBackColor = function(col){
	//col指定しないと透明
	NL_DebugLog.add("RealRect","@_setBackColor :" + col + this.div.id);
	if(NL_browser =='nn6'||NL_browser =='ie'||NL_browser =='opera'){
		this.div.style.backgroundColor=(!col)?'transparent':col;
	}
	else if(NL_browser =='nn4'){
		this.div.bgColor=(!col)?null:col;
	}
};
//*** その他 ***
NL_RealRect.prototype.copyTo = function(parent){
	//複製を返します//
	if(!parent){parent = document.body;}
	var Elem = new NL_RealRect(this._left,this._top,this._width,this._height,parent);

	NL_copyProperties(this,Elem);
	return Elem;
};
NL_RealRect.prototype.remove = function(){
	if(this.div){
		this.div.parentNode.removeChild(this.div);
		this.div = null;
	}
};

//******************** /NL_RealRect class ********************

