NL_importComplete=true;
//******************** NL_Rect class ********************
//概念上の四角形を管理するクラスです。
//表示可能な四角形を生成するには、NL_RealRect クラスを用いてください。

/*[使用法]
	
*/

//[使用例] example_NL_Canvas.htm

//[作者] Gen

//[メンバ一覧]
/****** global function ******
	NL_Rect(HTMLElement elem);			//HTML要素の位置とサイズから四角形を定義します
	NL_Rect(int left,int top,int width,int height);	//四角形を定義します
************************/

/****** property *******
	int	minW;		//
	int	minH;		//
	int	maxW;		//
	int	maxH;		//
	int	minX;		//
	int	minY;		//
	int	maxX;		//
	int	maxY;		//

	int	leftBasis;	//
	int	topBasis;	//
	int	widthBasis;	//
	int	heightBasis;	//

	int	_left;		//四角形の左辺の座標
	int	_top;		//四角形の上辺の座標
	int	_width;		//四角形の幅
	int	_height;	//四角形の高さ
**************************/

/****** method *******
	int	getLeft();			//四角形の左辺の座標を返します
	int	getRight();			//四角形の右辺の座標を返します
	int	getTop();			//四角形の上辺の座標を返します
	int	getBottom();			//四角形の下辺の座標を返します
	int	getWidth();			//四角形の幅を返します
	int	getHeight();			//四角形の高さを返します

	void	setPos(int left,int top,int width,int height);
						//四角形の配置を指定します。
	void	initBasis();			//現在の配置を基準に設定します。
	void	setPosFromBasis(int left,int top,int width,int height);
						//_initBasis()で指定した配置から相対的に配置を変更します
	void	moveTo(int left,int top);	//四角形の左辺をleftに上辺をtopに移動します
	void	moveBy(int dx,int dy);		//四角形を右にdx,下にdy移動します。
	void	resizeTo(int width,int height);//四角形の幅をwidth,高さをheightにします。
	void	resizeBy(int dw,int dh);	//四角形の幅をdw大きく高さをdh大きくします

	void	setSizeLimit(int minWidth,int minHeight,int maxWidth,int maxHeight);
						//四角形のサイズ制限値を設定
	void	setMoveLimit(int minX,int minY,int maxX,int maxY);
						//四角形の位置制限値を設定
************************/

//[プログラム本体]
//****** property *******
NL_Rect.prototype.minW=0;
NL_Rect.prototype.minH=0;
NL_Rect.prototype.maxW=Infinity;
NL_Rect.prototype.maxH=Infinity;
NL_Rect.prototype.minX=0;
NL_Rect.prototype.minY=0;
NL_Rect.prototype.maxX=Infinity;
NL_Rect.prototype.maxY=Infinity;

//********** constructor **********
function NL_Rect(arg1,arg2,arg3,arg4){
	var top,left,width,height;
	if(arguments.length>=4){
		left=arg1;
		top=arg2;
		width=arg3;
		height=arg4;
		if(NL_isNaN(left) || NL_isNaN(top) || NL_isNaN(width) || NL_isNaN(height))
			return NL_typeError('(left,top,width,height)','number');
	}
	else{
		var elem=arg1;
		if(!NL_isObject(elem) || !elem.style )
			return NL_typeError('elem','HTMLElement');
		left=__NL_Dom.getLeft(elem);
		top=__NL_Dom.getTop(elem);
		width=__NL_Dom.getWidth(elem);
		height=__NL_Dom.getHeight(elem);
	}
	this.setPos(left,top,width,height);
	this.initBasis();
}
//****** method *******
//*** 座標関連 ***
NL_Rect.prototype.getLeft = function(){
	return this._left();
};

NL_Rect.prototype.getTop = function(){
	return this._top;
};
NL_Rect.prototype.getRight = function(){
	return this.getLeft()+this.getWidth();
};

NL_Rect.prototype.getBottom = function(){
	return this.getTop()+this.getHeight();
};
NL_Rect.prototype.getWidth = function(){
	return this._width;
};
NL_Rect.prototype.getHeight = function(){
	return this._height;
};

//*** 移動リサイズ関連 ***
//絶対位置指定
NL_Rect.prototype.setPos = function(left,top,width,height){
	this.resizeTo(width,height);
	this.moveTo(left,top);
};
//相対位置指定(基点初期化)
NL_Rect.prototype.initBasis = function(){
	this.leftBasis=this.getLeft();
	this.topBasis=this.getTop();
	this.widthBasis=this.getWidth();
	this.heightBasis=this.getHeight();
};
//相対位置指定(現在値からの相対でなく基点からの相対)
NL_Rect.prototype.setPosFromBasis = function(dx,dy,dw,dh){
	if(!dw)	dw=0;
	if(!dh)	dh=0;
	if(!dx)	dx=0;
	if(!dy)	dy=0;
	
	var w=dw+this.widthBasis;
	var h=dh+this.heightBasis;
	var l=dx+this.leftBasis;
	var t=dy+this.topBasis;

	//リサイズ制限に引っかかったら、連動して移動も制限する。
	if(w<this.minW)	{if(dx)l=l-(this.minW-w);	w=this.minW;}
	if(h<this.minH)	{if(dy)t=t-(this.minH-h);	h=this.minH;}
	if(this.maxW<w)	{if(dx)l=l-(this.maxW-w);	w=this.maxW;}
	if(this.maxH<h)	{if(dy)t=t-(this.maxH-h);	h=this.maxH;}

	//移動制限に引っかかったら、連動してリサイズも制限する。
	if(l<this.minX)			{if(dw)w=w-(this.minX-l);	l=this.minX;}
	if(t<this.minY)			{if(dh)h=h-(this.minY-t);	t=this.minY;}
	if(this.maxX<l+this.getWidth())	{l=this.maxX-this.getWidth();}
	if(this.maxY<t+this.getHeight()){t=this.maxY-this.getHeight();}

	this.resizeTo(w,h);
	this.moveTo(l,t);
};

//絶対リサイズ
NL_Rect.prototype.resizeTo = function(width,height){
	if(NL_isNaN(width)) width=this.getWidth();
	if(NL_isNaN(height)) height=this.getHeight();

	if(width<this.minW) width=this.minW;
	if(height<this.minH) height=this.minH;
	if(this.maxW<width) width=this.maxW;
	if(this.maxH<height) height=this.maxH;

	this._width=width;
	this._height=height;
};
//相対リサイズ
NL_Rect.prototype.resizeBy = function(dw,dh){
	if(NL_isNaN(dw)) dw=0;
	if(NL_isNaN(dh)) dh=0;
	dw=Math.round(dw);
	dh=Math.round(dh);
	this.resizeTo(this.getWidth() +dw,this.getHeight()+dh);
};

//絶対移動
NL_Rect.prototype.moveTo = function(left,top){
	if(NL_isNaN(left)) left=this.getLeft();
	if(NL_isNaN(top)) top=this.getTop();

	if(left<this.minX) left=this.minX;
	if(top<this.minY) top=this.minY;
	if(this.maxX<left+this.getWidth()) left=this.maxX-this.getWidth();
	if(this.maxY<top+this.getHeight()) top=this.maxY-this.getHeight();

	this._left=left;
	this._top=top;
};
//相対移動
NL_Rect.prototype.moveBy = function(dx,dy){
	if(NL_isNaN(dx)) dx=0;
	if(NL_isNaN(dy)) dy=0;
	dx=Math.round(dx);
	dy=Math.round(dy);
	this.moveTo(this.getLeft()+dx, this.getHeight()+dy);
};
//サイズ制限値設定
NL_Rect.prototype.setSizeLimit = function(minWidth,minHeight,maxWidth,maxHeight){
	if(!NL_isNaN(minWidth))	this.minW=minWidth;
	if(!NL_isNaN(minHeight))this.minH=minHeight;
	if(!NL_isNaN(maxWidth))	this.maxW=maxWidth;
	if(!NL_isNaN(maxHeight))this.maxH=maxHeight;
};
//移動制限値設定
NL_Rect.prototype.setMoveLimit = function(minX,minY,maxX,maxY){
	if(!isNaN(minX) && minX!=null)this.minX=minX;
	if(!isNaN(minY) && minY!=null)this.minY=minY;
	if(!isNaN(maxX) && maxX!=null)this.maxX=maxX;
	if(!isNaN(maxY) && maxY!=null)this.maxY=maxY;
};
//******************** /NL_Rect class ********************

